/* Copyright 1998-2011 Research Foundation State University of New York */

/* This file is part of QuB.                                            */

/* QuB is free software; you can redistribute it and/or modify          */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* QuB is distributed in the hope that it will be useful,               */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License,   */
/* named LICENSE.txt, in the QuB program directory.  If not, see        */
/* <http://www.gnu.org/licenses/>.                                      */

#ifndef QUB_CURVE_EXPO_H
#define QUB_CURVE_EXPO_H

#include "QUB_BasicCurve.h"

typedef struct {
	double Amp, Tau;
} ExpoComponent;
static ExpoComponent EXPO_DEFAULTS = {1.0, 1.0};
#define EXPO_NPAR 2
static char EXPO_PARAM_NAMES[][MAX_NAMELEN] = {"Amp", "Tau"};

class QUB_Curve_Expo : public QUB_BasicCurve<ExpoComponent, EXPO_NPAR>
{
public:
	QUB_Curve_Expo(const char *name, IQUB_Curve *baselineCurve)
		: QUB_BasicCurve<ExpoComponent, EXPO_NPAR>(
				name,
				"Amp * e^(-(x-x0) / Tau)",
				&(EXPO_PARAM_NAMES[0][0]),
				EXPO_DEFAULTS,
				baselineCurve )
	{}
	virtual void evalOne(QF_Data *data, double *yy, int c)
	{
		double a = components[c].Amp;
		double ti = 1.0 / components[c].Tau;
		double mx0 = - getX0();
		int n = data->n;
		double *xx = data->xx;
		for (int i=0; i<n; ++i)
			yy[i] += a * exp(- (xx[i]+mx0) * ti);
	}
};

// make your own factory function with this signature,
// returning a new instance of your class:

// vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
IQUB_Curve* __stdcall CreateExpo();
IQUB_Curve* __stdcall CreateExpoLinear();
// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

#endif
